package com.ibm.oti.shared;

/*
 * Licensed Materials - Property of IBM,
 *     Copyright IBM Corp. 1998, 2010  All Rights Reserved
 */

import java.net.URL;

/**
 * SharedClassHelperFactory provides an interface
 * used to create various types of SharedClassHelper for ClassLoaders.
 * <p>
 * ClassLoaders and SharedClassHelpers have a one-to-one relationship.
 * Any attempts to get a helper for a ClassLoader that already has a different type of helper
 * will result in a HelperAlreadyDefinedException.
 * <p>
 * There are 3 diferent types of SharedClassHelper:
 * <p>
 * 1) SharedClassTokenHelper<p>
 * Store and find classes using a String token generated by the ClassLoader.
 * For use by ClassLoaders which require complete control over cache contents.
 * <p>
 * 2) SharedClassURLHelper<p>
 * Store and find classes using a URL location. Any URL may be used to store or find classes.<br>
 * For use by ClassLoaders which do not have the concept of a classpath, which load classes from multiple locations.<br>
 * Classes may only be stored using jar/zip or file URLs. Classes are automatically kept up-to-date by the cache.<br>
 * Classes stored using SharedClassURLClasspathHelper may be found using this helper and vice-versa.
 * <p>
 * 3) SharedClassURLClasspathHelper<p>
 * Store and find classes using a URL classpath. URLs may be appended to the classpath at any time.
 * The classpath may also be modified under certain cirumstances (see SharedClassURLClasspathHelper javadoc).<br>
 * For use by ClassLoaders which load classes using a URL classpath.<br>
 * Classes may only be stored using jar/zip or file URLs. Classes are automatically kept up-to-date by the cache.<br>
 * Classes stored using SharedClassURLHelper may be found using this helper and vice-versa.
 * <p>
 * @author		OTI
 * @version		initial
 *
 * @see SharedClassTokenHelper
 * @see SharedClassURLHelper
 * @see SharedClassURLClasspathHelper
 */
public interface SharedClassHelperFactory {

	/**
	 * Return a SharedClassTokenHelper for a given ClassLoader.<p>
	 * Creates a new SharedClassTokenHelper if one cannot be found, otherwise returns existing SharedClassTokenHelper.<br>
	 * Throws a HelperAlreadyDefinedException if the ClassLoader already has a different type of helper.<br>
	 * Returns null if a SecurityManager is installed and there is no
	 * SharedClassPermission for the ClassLoader specified.
	 * <p>
	 * @see SharedClassTokenHelper
	 *
	 * @author 		OTI
	 * @version 	initial
	 *
	 * @param 		loader ClassLoader.
	 * 					ClassLoader to which this SharedClassTokenHelper will belong
	 *
	 * @return		SharedClassTokenHelper.
	 * 					A new or existing SharedClassTokenHelper
	 *
	 * @throws 		HelperAlreadyDefinedException
	 */
	public SharedClassTokenHelper getTokenHelper(ClassLoader loader) throws HelperAlreadyDefinedException;

	/**
	 * Return a SharedClassTokenHelper for a given ClassLoader.<p>
	 * Creates a new SharedClassTokenHelper if one cannot be found, otherwise returns existing SharedClassTokenHelper.<br>
	 * Throws a HelperAlreadyDefinedException if the ClassLoader already has a different type of helper.<br>
	 * Returns null if a SecurityManager is installed and there is no
	 * SharedClassPermission for the ClassLoader specified.
	 * <p>
	 * @see SharedClassTokenHelper
	 *
	 * @author 		OTI
	 * @version 	initial
	 *
	 * @param 		loader ClassLoader.
	 * 					ClassLoader to which this SharedClassTokenHelper will belong
	 *
	 * @return		SharedClassTokenHelper.
	 * 					A new or existing SharedClassTokenHelper
	 *
	 * @param 		filter SharedClassURLFilter.
	 * 					Specify a filter which limits the classes that are found or stored in the cache
	 *
	 * @throws 		HelperAlreadyDefinedException
	 */
	public SharedClassTokenHelper getTokenHelper(ClassLoader loader, SharedClassFilter filter) throws HelperAlreadyDefinedException;

	/**
	 * Return a SharedClassURLHelper for a given ClassLoader.<p>
	 * Creates a new SharedClassURLHelper if one cannot be found, otherwise returns existing SharedClassURLHelper.<br>
	 * Throws a HelperAlreadyDefinedException if the ClassLoader already has a different type of helper.<br>
	 * Returns null if a SecurityManager is installed and there is no
	 * SharedClassPermission for the ClassLoader specified.
	 * <p>
	 * @see SharedClassURLHelper
	 *
	 * @author 		OTI
	 * @version 	initial
	 *
	 * @param 		loader ClassLoader.
	 * 					ClassLoader to which this SharedClassURLHelper will belong
	 *
	 * @return		SharedClassURLHelper.
	 * 					A new or existing SharedClassURLHelper
	 *
	 * @throws 		HelperAlreadyDefinedException
	 */
	public SharedClassURLHelper getURLHelper(ClassLoader loader) throws HelperAlreadyDefinedException;

	/**
	 * Return a SharedClassURLClasspathHelper for a given ClassLoader.<p>
	 * Creates a new SharedClassURLClasspathHelper if one cannot be found,
	 *   otherwise if the classpath specified matches the classpath of an existing helper, returns existing SharedClassURLClasspathHelper.<br>
	 * Throws a HelperAlreadyDefinedException if the ClassLoader already has a different type of helper OR
	 *   if the ClassLoader has a SharedClassURLClasspathHelper with a different classpath.<br>
	 * Returns null if a SecurityManager is installed and there is no
	 * SharedClassPermission for the ClassLoader specified.
	 * <p>
	 * @see SharedClassURLClasspathHelper
	 *
	 * @author 		OTI
	 * @version 	initial
	 *
	 * @param 		loader ClassLoader.
	 * 					ClassLoader to which this SharedClassURLClasspathHelper will belong
	 *
	 * @param 		classpath URL[].
	 * 					The current URL classpath of this ClassLoader
	 *
	 * @return		SharedClassURLClasspathHelper.
	 * 					A new or existing SharedClassURLClasspathHelper
	 *
	 * @throws 		HelperAlreadyDefinedException
	 */
	public SharedClassURLClasspathHelper getURLClasspathHelper(ClassLoader loader, URL[] classpath) throws HelperAlreadyDefinedException;

	/**
	 * Return a SharedClassURLClasspathHelper for a given ClassLoader.<p>
	 * Creates a new SharedClassURLClasspathHelper if one cannot be found,
	 *   otherwise if the classpath specified matches the classpath of an existing helper, returns existing SharedClassURLClasspathHelper.<br>
	 * If a new SharedClassHelper is created, the specified SharedClassURLFilter is applied to it. If the filter argument is null, no filter is applied.<br>
	 * Throws a HelperAlreadyDefinedException if the ClassLoader already has a different type of helper OR
	 *   if the ClassLoader has a SharedClassURLClasspathHelper with a different classpath.<br>
	 * Returns null if a SecurityManager is installed and there is no
	 * SharedClassPermission for the ClassLoader specified.
	 * <p>
	 * @see SharedClassURLClasspathHelper
	 * @see SharedClassFilter
	 *
	 * @author 		OTI
	 * @version 	initial
	 *
	 * @param 		loader ClassLoader.
	 * 					ClassLoader to which this SharedClassURLClasspathHelper will belong
	 *
	 * @param 		classpath URL[].
	 * 					The current URL classpath of this ClassLoader
	 *
	 * @param 		filter SharedClassURLFilter.
	 * 					Specify a filter which limits the classes that are found or stored in the cache
	 *
	 * @return		SharedClassURLClasspathHelper.
	 * 					A new or existing SharedClassURLClasspathHelper
	 *
	 * @throws 		HelperAlreadyDefinedException
	 */
	public SharedClassURLClasspathHelper getURLClasspathHelper(ClassLoader loader, URL[] classpath, SharedClassFilter filter) throws HelperAlreadyDefinedException;

	/**
	 * Return a SharedClassURLHelper for a given ClassLoader.<p>
	 * Creates a new SharedClassURLHelper if one cannot be found, otherwise returns existing SharedClassURLHelper.<br>
	 * If a new SharedClassHelper is created, the specified SharedClassURLFilter is applied to it. If the filter argument is null, no filter is applied.<br>
	 * Throws a HelperAlreadyDefinedException if the ClassLoader already has a different type of helper.<br>
	 * Returns null if a SecurityManager is installed and there is no
	 * SharedClassPermission for the ClassLoader specified.
	 * <p>
	 * @see SharedClassURLHelper
	 * @see SharedClassFilter
	 *
	 * @author 		OTI
	 * @version 	initial
	 *
	 * @param 		loader ClassLoader.
	 * 					ClassLoader to which this SharedClassURLHelper will belong
	 *
	 * @param 		filter SharedClassURLFilter.
	 * 					Specify a filter which limits the classes that are found or stored in the cache
	 *
	 * @return		SharedClassURLHelper.
	 * 					A new or existing SharedClassURLHelper
	 *
	 * @throws 		HelperAlreadyDefinedException
	 */
	public SharedClassURLHelper getURLHelper(ClassLoader loader, SharedClassFilter filter) throws HelperAlreadyDefinedException;

	/**
	 * Utility function which returns a SharedClassHelper for a given ClassLoader.<p>
	 * Can be used to determine whether a given ClassLoader already has a helper, before calling a getter method.<br>
	 * Returns an existing SharedClassHelper or null.
	 * <p>
	 * @see SharedClassHelper
	 *
	 * @author 		OTI
	 * @version 	initial
	 *
	 * @param 		loader ClassLoader.
	 * 					ClassLoader which may or may not have a SharedClassHelper
	 *
	 * @return		SharedClassHelper.
	 * 					A helper if one exists for this ClassLoader or null otherwise.
	 */
	public SharedClassHelper findHelperForClassLoader(ClassLoader loader);
}
