
package com.ibm.tools.attach.javaSE;

/*
 * Licensed Materials - Property of IBM,
 *     Copyright IBM Corp. 2009, 2010  All Rights Reserved
 */

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.InetAddress;
import java.net.Socket;
import java.util.Properties;

/**
 * This class handles established connections initiated by another VM
 *
 */
class Attachment extends Thread {

	/**
	 * constants for communication between Attachment and VirtualMachine
	 */

	/**
	 * List of exceptions encountered
	 */
	public Exception lastError;
	private OutputStream responseStream;
	private Socket attacherSocket;
	private final int portNumber;
	private InputStream commandStream;
	private String attachError;
	private final AttachHandler handler;
	private final String key;

	/**
	 * @param attachHandler
	 *            main handler object for this VM
	 * @param rc
	 *            information to connect to attacher
	 */
	public Attachment(AttachHandler attachHandler, Reply rc) {
		setName("Attachment " + rc.getPortNumber()); //$NON-NLS-1$
		portNumber = rc.getPortNumber();
		this.key = rc.getKey();
		this.handler = attachHandler;

		setDaemon(true);
	}

	/**
	 * Create an attachment with a socket connection to the attacher
	 *
	 * @param portNum
	 *            for socket to connect to attacher
	 * @return true if successfully connected
	 */
	public boolean connectToAttacher(int portNum) {

		try {
			IPC.logMessage("connectToAttacher portNum=", portNum); //$NON-NLS-1$
			InetAddress localHost = InetAddress.getLocalHost();
			attacherSocket = new Socket(localHost, portNum);
			IPC.logMessage("connectToAttacher localPort=",  attacherSocket.getLocalPort(), " remotePort=", Integer.toString(attacherSocket.getPort())); //$NON-NLS-1$//$NON-NLS-2$
			responseStream = attacherSocket.getOutputStream();
			commandStream = attacherSocket.getInputStream();
			AttachmentConnection.streamSend(responseStream, Response.CONNECTED + ' '
					+ key + ' ');
		} catch (IOException e) {
			IPC.logMessage("connectToAttacher exception " + e.getMessage() + " " + e.toString()); //$NON-NLS-1$ //$NON-NLS-2$
			try {
				if (null != responseStream) {
					responseStream.close();
				}
				if (null != commandStream) {
					commandStream.close();
				}
				if (null != attacherSocket) {
					attacherSocket.close();
				}
			} catch (IOException e1) {
				return false;
			}
			return false;
		} catch (Exception otherException) {
			IPC.logMessage("connectToAttacher exception ", otherException.toString()); //$NON-NLS-1$
			return false;
		}
		return true;
	}

	@Override
	public void run() {
		boolean terminate = false;
		IPC.logMessage("Attachment run"); //$NON-NLS-1$
		connectToAttacher(getPortNumber());
		while (!terminate && !isInterrupted()) {
			terminate = doCommand(commandStream, responseStream);
		}
		try {
			AttachmentConnection.streamSend(responseStream, Response.DETACHED);
			if (null != responseStream) {
				responseStream.close();
				responseStream = null;
			}
			if (null != commandStream) {
				commandStream.close();
				commandStream = null;
			}
		} catch (IOException e) {
			lastError = e;
		}
		if (null != handler) {
			handler.removeAttachment(this);
		}
	}

	/**
	 * @param cmdStream
	 *            channel for commands from attacher
	 * @param respStream
	 *            channel to acknowledge commands This is called by a native
	 *            function when requested by the attacher.
	 * @return true on error, detach command, or command socket closed
	 */
	public boolean doCommand(InputStream cmdStream, OutputStream respStream) {
		try {
			String cmd = AttachmentConnection.streamReceiveString(cmdStream);
			IPC.logMessage("doCommand ", cmd); //$NON-NLS-1$

			if (null == cmd) {
				return true;
			}
			if (cmd.startsWith(Command.DETACH)) {
				return true;
			} else if (cmd.startsWith(Command.LOADAGENT)) {
				if (parseLoadAgent(cmd)) {
					AttachmentConnection.streamSend(respStream, Response.ACK);
				} else {
					AttachmentConnection.streamSend(respStream, Response.ERROR
							+ " " + attachError); //$NON-NLS-1$
				}
			} else if (cmd.startsWith(Command.GET_SYSTEM_PROPERTIES)) {
				replyWithProperties(System.getProperties());
			} else if (cmd.startsWith(Command.GET_AGENT_PROPERTIES)) {
				replyWithProperties(AttachHandler.getAgentProperties());
			} else {
				AttachmentConnection.streamSend(respStream, Response.ERROR
						+ " command invalid: " + cmd); //$NON-NLS-1$
			}
		} catch (IOException e) {
			IPC.logMessage("doCommand exception ", e.toString()); //$NON-NLS-1$
			return true;
		}
		return false;
	}

	private void replyWithProperties(Properties props) throws IOException {
		ByteArrayOutputStream propsBuffer = new ByteArrayOutputStream();
		props.store(propsBuffer, ""); //$NON-NLS-1$
		responseStream.write(propsBuffer.toByteArray());
		responseStream.write(0);
	}

	/**
	 * close socket and other cleanup
	 */
	public synchronized void teardown() {
		try {
			if (null != responseStream) {
				responseStream.close();
			}
			if (null != attacherSocket) {
				attacherSocket.close();
			}
			if (null != commandStream) {
				commandStream.close();
			}
		} catch (IOException e) {
			lastError = e;
		}
	}

	/**
	 * parse and execute a loadAgent command. Set the error string if there is a
	 * problem
	 *
	 * @param cmd
	 *            command string of the form
	 *            "ATTACH_LOADAGENT(agentname,options)" or
	 *            "ATTACH_LOADAGENT(agentname)"
	 * @return true if no error
	 */
	private boolean parseLoadAgent(String cmd) {
		int openParenIndex = cmd.indexOf('(');
		int closeParenIndex = cmd.lastIndexOf(')');
		int commaIndex = cmd.indexOf(',');
		String optionString = ""; //$NON-NLS-1$
		boolean decorate = true;

		attachError = null;
		if (cmd.startsWith(Command.LOADAGENTPATH)) {
			decorate = false;
		}
		if ((openParenIndex < 0) || (closeParenIndex < 0)) {
			attachError = "syntax error"; //$NON-NLS-1$
			return false;
		}
		int agentNameEnd = (commaIndex < 0) ? closeParenIndex : commaIndex;
		String agentName = cmd.substring(openParenIndex + 1, agentNameEnd);
		if (agentName.length() < 1) {
			attachError = "invalid agent name"; //$NON-NLS-1$
			return false;
		}
		if (commaIndex > 0) {
			optionString = cmd.substring(commaIndex + 1, closeParenIndex);
		}
		attachError = loadAgentLibrary(agentName, optionString, decorate);
		if (attachError != null) {
			return false;
		}

		return true;
	}

	/**
	 *
	 * @param agentLibrary
	 *            name of the agent library
	 * @param options
	 *            arguments to the library's Agent_OnAttach function
	 * @param decorate
	 *            add prefixes and suffixes to the library name.
	 * @return null if successful, diagnostic string if error
	 */
	public String loadAgentLibrary(String agentLibrary, String options,
			boolean decorate) {
		IPC.logMessage("loadAgentLibrary " + agentLibrary + ':' + options + " decorate=" + decorate); //$NON-NLS-1$ //$NON-NLS-2$
		ClassLoader loader = java.lang.ClassLoader.getSystemClassLoader();
		int status = loadAgentLibraryImpl(loader ,agentLibrary,  options, decorate);
		if (0 != status) {
			if (-1 == status) {
				return Response.EXCEPTION_AGENT_LOAD_EXCEPTION + ' '
						+ agentLibrary + ':' + options;
			} else {
				return Response.EXCEPTION_AGENT_INITIALIZATION_EXCEPTION
						+ status;
			}
		}
		return null;
	}

	/**
	 *
	 * @param agentLibrary
	 *            name of the agent library
	 * @param options
	 *            arguments to the library's Agent_OnAttach function
	 * @param decorate
	 *            add prefixes and suffixes to the library name.
	 * @return 0 if all went well
	 */
	private native int loadAgentLibraryImpl(ClassLoader loader,String agentLibrary,
			String options, boolean decorate);

	private int getPortNumber() {
		return portNumber;
	}
}
